""" Helpers for calling commands, capture their output, return result code

Subprocess library just does not provide all the simplicity we would like

Stay python2 compatible --> no timeouts

.. codeauthor:: Christian Herdtweck, christian.herdtweck@intra2net.com
"""

from subprocess import Popen, PIPE


def call_and_capture(command, stdin_data=None, split_lines=True,
                     *args, **kwargs):
    """ call command, captures stdout, stderr and return code, return when done

    Use only for commands with little output since all output has to be
    buffered!
    Quoting :py:mod:`subprocess`:

    ..note:: The data read is buffered in memory, so do not use this method if
             the data size is large or unlimited.

    Forwards all args to Popen constructor, except:
    stdout=PIPE (forced, ignored if in kwargs)
    stderr=PIPE (forced, ignored if in kwargs)
    shell=False (except if set in kwargs)
    universal_newlines=True (except if set in kwargs)

    :param command: forwarded as first arg to Popen constructor
    :param str stdin_data: forwarded to stdin of process through communicate
    :param bool split_lines: True (default) to split output line-wise and
                             return list of strings; False --> return single
                             string for out and one for err
    :param args: forwarded to Popen constructor
    :param kwargs: forwarded to Popen constructor
    :returns: (return_code, stdout, stderr); stdout and stderr are lists of
        text lines (without terminating newlines); if universal_newlines is
        True (default), the lines are of type str, otherwise they are non-
        unicode text (type str in py2, bytes in py3). If split_lines is False
        (not default), then stdout and stderr are single multi-line strings

    :raise: OSError (e.g., if command does not exist), ValueError if args are
            invalid; no :py:class:`subprocess.CalledProcessError` nor
            :py:class:`subprocess.TimeoutExpired`
    """

    # construct args
    enforced_params = ('stdout', 'stderr')
    my_kwargs = dict(stdout=PIPE, stderr=PIPE,
                     shell=False, universal_newlines=True)
    for key, value in kwargs.items():
        if key not in enforced_params:
            my_kwargs[key] = value

    # run command
    proc = Popen(command, *args, **my_kwargs)
    stdout_data, stderr_data = proc.communicate(stdin_data)

    # return
    if split_lines:
        return proc.returncode, stdout_data.splitlines(), \
            stderr_data.splitlines()
    else:
        return proc.returncode, stdout_data, stderr_data
